@@ -1,11 +1,13 @@
+"""Support for Netatmo security devices (cameras, smoke detectors, sirens, window sensors, events and persons)."""
 import imghdr
 import time
+from abc import ABC
 from collections import defaultdict
 from typing import Any, Dict, List, Optional, Tuple
 
 from requests.exceptions import ReadTimeout
 
-from .auth import NetatmoOAuth2
+from .auth import AbstractAsyncAuth, NetatmoOAuth2
 from .exceptions import ApiError, NoDevice
 from .helpers import _BASE_URL, LOG
 
@@ -17,46 +19,24 @@ _SETPERSONSHOME_REQ = _BASE_URL + "api/setpersonshome"
 _SETSTATE_REQ = _BASE_URL + "api/setstate"
 
 
-class CameraData:
-    """
-    Class of Netatmo camera informations
-        (Homes, cameras, smoke detectors, modules, events, persons)
-    """
+class AbstractCameraData(ABC):
+    """Abstract class of Netatmo camera data."""
 
-    def __init__(self, auth: NetatmoOAuth2, size: int = 30) -> None:
-        """Initialize self.
+    raw_data: Dict = defaultdict(dict)
+    homes: Dict = defaultdict(dict)
+    persons: Dict = {}
+    events: Dict = defaultdict(dict)
+    outdoor_events: Dict = defaultdict(dict)
+    cameras: Dict = defaultdict(dict)
+    smoke_detectors: Dict = defaultdict(dict)
+    modules: Dict = {}
+    last_event: Dict = {}
+    outdoor_last_event: Dict = {}
+    types: Dict = defaultdict(dict)
 
-        Arguments:
-            auth {NetatmoOAuth2} -- Authentication information with a valid access token
-
-        Keyword Arguments:
-            size {int} -- Number of events to retrieve. (default: {30})
-
-        Raises:
-            NoDevice: No devices found.
-        """
-        self.auth = auth
-
-        post_params = {"size": size}
-        resp = self.auth.post_request(url=_GETHOMEDATA_REQ, params=post_params)
-        if resp is None or "body" not in resp:
-            raise NoDevice("No device data returned by Netatmo server")
-
-        self.raw_data = resp["body"].get("homes")
-        if not self.raw_data:
-            raise NoDevice("No device data available")
-
-        self.homes: Dict = {d["id"]: d for d in self.raw_data}
-
-        self.persons: Dict = {}
-        self.events: Dict = defaultdict(dict)
-        self.outdoor_events: Dict = defaultdict(dict)
-        self.cameras: Dict = defaultdict(dict)
-        self.smokedetectors: Dict = defaultdict(dict)
-        self.modules: Dict = {}
-        self.last_event: Dict = {}
-        self.outdoor_last_event: Dict = {}
-        self.types: Dict = defaultdict(dict)
+    def process(self) -> None:
+        """Process data from API."""
+        self.homes = {d["id"]: d for d in self.raw_data}
 
         for item in self.raw_data:
             home_id: str = item.get("id", "")
@@ -81,15 +61,9 @@ class CameraData:
                         self.modules[module["id"]]["cam_id"] = camera["id"]
 
             for smoke in item.get("smokedetectors", []):
-                self.smokedetectors[home_id][smoke["id"]] = smoke
+                self.smoke_detectors[home_id][smoke["id"]] = smoke
                 self.types[home_id][smoke["type"]] = smoke
 
-        self._store_last_event()
-
-        for home_id in self.homes:
-            for camera_id in self.cameras[home_id]:
-                self.update_camera_urls(camera_id)
-
     def _store_events(self, event):
         if event["type"] == "outdoor":
             self.outdoor_events[event["camera_id"]][event["time"]] = event
@@ -122,9 +96,9 @@ class CameraData:
 
     def get_smokedetector(self, smoke_id: str) -> Optional[dict]:
         """Get smoke detector."""
-        for home_id in self.smokedetectors:
-            if smoke_id in self.smokedetectors[home_id]:
-                return self.smokedetectors[home_id][smoke_id]
+        for home_id in self.smoke_detectors:
+            if smoke_id in self.smoke_detectors[home_id]:
+                return self.smoke_detectors[home_id][smoke_id]
 
         return None
 
@@ -136,37 +110,6 @@ class CameraData:
         camera_data = self.get_camera(camera_id)
         return camera_data.get("vpn_url", None), camera_data.get("local_url", None)
 
-    def update_camera_urls(self, camera_id: str) -> None:
-        """Update and validate the camera urls."""
-        camera_data = self.get_camera(camera_id)
-        home_id = camera_data["home_id"]
-
-        if not camera_data or camera_data.get("status") == "disconnected":
-            self.cameras[home_id][camera_id]["local_url"] = None
-            self.cameras[home_id][camera_id]["vpn_url"] = None
-            return
-
-        vpn_url = camera_data.get("vpn_url")
-        if vpn_url and camera_data.get("is_local"):
-
-            def check_url(url: str) -> Optional[str]:
-                try:
-                    resp = self.auth.post_request(url=f"{url}/command/ping")
-                except ReadTimeout:
-                    LOG.debug("Timeout validation of camera url %s", url)
-                    return None
-                except ApiError:
-                    LOG.debug("Api error for camera url %s", url)
-                    return None
-
-                return resp.get("local_url") if resp else None
-
-            temp_local_url = check_url(vpn_url)
-            if temp_local_url:
-                self.cameras[home_id][camera_id]["local_url"] = check_url(
-                    temp_local_url,
-                )
-
     def get_light_state(self, camera_id: str) -> Optional[str]:
         """Return the current mode of the floodlight of a presence camera."""
         camera_data = self.get_camera(camera_id)
@@ -184,72 +127,16 @@ class CameraData:
             if "pseudo" in person and not person["out_of_sight"]
         ]
 
-    def set_persons_home(self, person_ids: List[str], home_id: str):
-        """Mark persons as home.
-
-        Arguments:
-            person_ids {list} -- IDs of persons
-            home_id {str} -- ID of a home
-        """
-        post_params = {"home_id": home_id, "person_ids[]": person_ids}
-        return self.auth.post_request(url=_SETPERSONSHOME_REQ, params=post_params)
-
-    def set_persons_away(self, person_id: str, home_id: str):
-        """Mark a person as away or set the whole home to being empty.
-
-        Arguments:
-            person_id {str} -- ID of a person
-            home_id {str} -- ID of a home
-        """
-        post_params = {"home_id": home_id, "person_id": person_id}
-        return self.auth.post_request(url=_SETPERSONSAWAY_REQ, params=post_params)
-
     def get_person_id(self, name: str) -> Optional[str]:
-        """Retrieve the ID of a person.
-
-        Arguments:
-            name {str} -- Name of a person
-
-        Returns:
-            str -- ID of a person
-        """
+        """Retrieve the ID of a person."""
         for pid, data in self.persons.items():
             if name == data.get("pseudo"):
                 return pid
 
         return None
 
-    def get_camera_picture(
-        self,
-        image_id: str,
-        key: str,
-    ) -> Tuple[bytes, Optional[str]]:
-        """Download a specific image (of an event or user face) from the camera."""
-        post_params = {"image_id": image_id, "key": key}
-        resp = self.auth.post_request(url=_GETCAMERAPICTURE_REQ, params=post_params)
-        image_type = imghdr.what("NONE.FILE", resp)
-        return resp, image_type
-
-    def get_profile_image(self, name: str) -> Tuple[Optional[bytes], Optional[str]]:
-        """Retrieve the face of a given person."""
-        for person in self.persons:
-            if name == self.persons[person].get("pseudo"):
-                image_id = self.persons[person]["face"]["id"]
-                key = self.persons[person]["face"]["key"]
-                return self.get_camera_picture(image_id, key)
-
-        return None, None
-
-    def update_events(
-        self,
-        home_id: str,
-        event_id: str = None,
-        device_type: str = None,
-    ) -> None:
-        """Update the list of events."""
-        # Either event_id or device_type must be given
-        if not (event_id or device_type):
-            raise ApiError
+    def build_event_id(self, event_id: Optional[str], device_type: Optional[str]):
+        """Build event id."""
 
         def get_event_id(data: Dict):
             events = {e["time"]: e for e in data.values()}
@@ -266,27 +153,7 @@ class CameraData:
                 # for the Presence camera and for the smoke detector
                 event_id = get_event_id(self.outdoor_last_event)
 
-        post_params = {"home_id": home_id, "event_id": event_id}
-
-        event_list: List = []
-        resp: Optional[Dict[str, Any]] = None
-        try:
-            resp = self.auth.post_request(url=_GETEVENTSUNTIL_REQ, params=post_params)
-            if resp is not None:
-                event_list = resp["body"]["events_list"]
-        except ApiError:
-            pass
-        except KeyError:
-            if resp is not None:
-                LOG.debug("event_list response: %s", resp)
-                LOG.debug("event_list body: %s", dict(resp)["body"])
-            else:
-                LOG.debug("No resp received")
-
-        for event in event_list:
-            self._store_events(event)
-
-        self._store_last_event()
+        return event_id
 
     def person_seen_by_camera(
         self,
@@ -511,24 +378,14 @@ class CameraData:
 
         return False
 
-    def set_state(
+    def build_state_params(
         self,
         camera_id: str,
-        home_id: str = None,
-        floodlight: str = None,
-        monitoring: str = None,
-    ) -> bool:
-        """Turn camera (light) on/off.
-
-        Arguments:
-            camera_id {str} -- ID of a camera
-            home_id {str} -- ID of a home
-            floodlight {str} -- Mode for floodlight (on/off/auto)
-            monitoring {str} -- Mode for monitoring (on/off)
-
-        Returns:
-            Boolean -- Success of the request
-        """
+        home_id: Optional[str],
+        floodlight: Optional[str],
+        monitoring: Optional[str],
+    ):
+        """."""
         if home_id is None:
             home_id = self.get_camera(camera_id)["home_id"]
 
@@ -548,7 +405,99 @@ class CameraData:
             else:
                 module[param] = val
 
-        post_params = {"json": {"home": {"id": home_id, "modules": [module]}}}
+        return {"id": home_id, "modules": [module]}
+
+
+class CameraData(AbstractCameraData):
+    """Class of Netatmo camera data."""
+
+    def __init__(self, auth: NetatmoOAuth2) -> None:
+        """Initialize the Netatmo camera data.
+
+        Arguments:
+            auth {NetatmoOAuth2} -- Authentication information with a valid access token
+        """
+        self.auth = auth
+
+    def update(self, events: int = 30) -> None:
+        """Fetch and process data from API."""
+        resp = self.auth.post_request(url=_GETHOMEDATA_REQ, params={"size": events})
+        if resp is None or "body" not in resp:
+            raise NoDevice("No device data returned by Netatmo server")
+
+        self.raw_data = resp["body"].get("homes")
+        if not self.raw_data:
+            raise NoDevice("No device data available")
+
+        self.process()
+        self._update_all_camera_urls()
+        self._store_last_event()
+
+    def _update_all_camera_urls(self) -> None:
+        """Update all camera urls."""
+        for home_id in self.homes:
+            for camera_id in self.cameras[home_id]:
+                self.update_camera_urls(camera_id)
+
+    def update_camera_urls(self, camera_id: str) -> None:
+        """Update and validate the camera urls."""
+        camera_data = self.get_camera(camera_id)
+        home_id = camera_data["home_id"]
+
+        if not camera_data or camera_data.get("status") == "disconnected":
+            self.cameras[home_id][camera_id]["local_url"] = None
+            self.cameras[home_id][camera_id]["vpn_url"] = None
+            return
+
+        vpn_url = camera_data.get("vpn_url")
+        if vpn_url and camera_data.get("is_local"):
+
+            def check_url(url: str) -> Optional[str]:
+                try:
+                    resp = self.auth.post_request(url=f"{url}/command/ping")
+                except ReadTimeout:
+                    LOG.debug("Timeout validation of camera url %s", url)
+                    return None
+                except ApiError:
+                    LOG.debug("Api error for camera url %s", url)
+                    return None
+
+                return resp.get("local_url") if resp else None
+
+            temp_local_url = check_url(vpn_url)
+            if temp_local_url:
+                self.cameras[home_id][camera_id]["local_url"] = check_url(
+                    temp_local_url,
+                )
+
+    def set_state(
+        self,
+        camera_id: str,
+        home_id: str = None,
+        floodlight: str = None,
+        monitoring: str = None,
+    ) -> bool:
+        """Turn camera (light) on/off.
+
+        Arguments:
+            camera_id {str} -- ID of a camera
+            home_id {str} -- ID of a home
+            floodlight {str} -- Mode for floodlight (on/off/auto)
+            monitoring {str} -- Mode for monitoring (on/off)
+
+        Returns:
+            Boolean -- Success of the request
+        """
+        post_params = {
+            "json": {
+                "home": self.build_state_params(
+                    camera_id,
+                    home_id,
+                    floodlight,
+                    monitoring,
+                ),
+            },
+        }
 
         try:
             resp = self.auth.post_request(url=_SETSTATE_REQ, params=post_params)
@@ -562,3 +511,233 @@ class CameraData:
 
         LOG.debug("%s", resp)
         return True
+
+    def set_persons_home(self, person_ids: List[str], home_id: str):
+        """Mark persons as home."""
+        post_params = {"home_id": home_id, "person_ids[]": person_ids}
+        return self.auth.post_request(url=_SETPERSONSHOME_REQ, params=post_params)
+
+    def set_persons_away(self, person_id: str, home_id: str):
+        """Mark a person as away or set the whole home to being empty."""
+        post_params = {"home_id": home_id, "person_id": person_id}
+        return self.auth.post_request(url=_SETPERSONSAWAY_REQ, params=post_params)
+
+    def get_camera_picture(
+        self,
+        image_id: str,
+        key: str,
+    ) -> Tuple[bytes, Optional[str]]:
+        """Download a specific image (of an event or user face) from the camera."""
+        post_params = {"image_id": image_id, "key": key}
+        resp = self.auth.post_request(url=_GETCAMERAPICTURE_REQ, params=post_params)
+        image_type = imghdr.what("NONE.FILE", resp)
+        return resp, image_type
+
+    def get_profile_image(self, name: str) -> Tuple[Optional[bytes], Optional[str]]:
+        """Retrieve the face of a given person."""
+        for person in self.persons:
+            if name == self.persons[person].get("pseudo"):
+                image_id = self.persons[person]["face"]["id"]
+                key = self.persons[person]["face"]["key"]
+                return self.get_camera_picture(image_id, key)
+
+        return None, None
+
+    def update_events(
+        self,
+        home_id: str,
+        event_id: str = None,
+        device_type: str = None,
+    ) -> None:
+        """Update the list of events."""
+        if not (event_id or device_type):
+            raise ApiError
+
+        post_params = {
+            "home_id": home_id,
+            "event_id": self.build_event_id(event_id, device_type),
+        }
+
+        event_list: List = []
+        resp: Optional[Dict[str, Any]] = None
+        try:
+            resp = self.auth.post_request(url=_GETEVENTSUNTIL_REQ, params=post_params)
+            if resp is not None:
+                event_list = resp["body"]["events_list"]
+        except ApiError:
+            pass
+        except KeyError:
+            if resp is not None:
+                LOG.debug("event_list response: %s", resp)
+                LOG.debug("event_list body: %s", dict(resp)["body"])
+            else:
+                LOG.debug("No resp received")
+
+        for event in event_list:
+            self._store_events(event)
+
+        self._store_last_event()
+
+
+class AsyncCameraData(AbstractCameraData):
+    """Class of Netatmo camera data."""
+
+    def __init__(self, auth: AbstractAsyncAuth) -> None:
+        """Initialize the Netatmo camera data.
+
+        Arguments:
+            auth {AbstractAsyncAuth} -- Authentication information with a valid access token
+        """
+        self.auth = auth
+
+    async def async_update(self, events: int = 30) -> None:
+        """Fetch and process data from API."""
+        resp = await self.auth.async_post_request(
+            url=_GETHOMEDATA_REQ,
+            params={"size": events},
+        )
+        if resp is None or "body" not in resp:
+            raise NoDevice("No device data returned by Netatmo server")
+
+        self.raw_data = resp["body"].get("homes")
+        if not self.raw_data:
+            raise NoDevice("No device data available")
+
+        self.process()
+        await self._async_update_all_camera_urls()
+        self._store_last_event()
+
+    async def _async_update_all_camera_urls(self) -> None:
+        """Update all camera urls."""
+        for home_id in self.homes:
+            for camera_id in self.cameras[home_id]:
+                await self.async_update_camera_urls(camera_id)
+
+    async def async_set_state(
+        self,
+        camera_id: str,
+        home_id: str = None,
+        floodlight: str = None,
+        monitoring: str = None,
+    ) -> bool:
+        """Turn camera (light) on/off.
+
+        Arguments:
+            camera_id {str} -- ID of a camera
+            home_id {str} -- ID of a home
+            floodlight {str} -- Mode for floodlight (on/off/auto)
+            monitoring {str} -- Mode for monitoring (on/off)
+
+        Returns:
+            Boolean -- Success of the request
+        """
+        post_params = {
+            "json": {
+                "home": self.build_state_params(
+                    camera_id,
+                    home_id,
+                    floodlight,
+                    monitoring,
+                ),
+            },
+        }
+
+        try:
+            resp = await self.auth.async_post_request(
+                url=_SETSTATE_REQ,
+                params=post_params,
+            )
+        except ApiError as err_msg:
+            LOG.error("%s", err_msg)
+            return False
+
+        if "error" in resp:
+            LOG.debug("%s", resp)
+            return False
+
+        LOG.debug("%s", resp)
+        return True
+
+    async def async_update_camera_urls(self, camera_id: str) -> None:
+        """Update and validate the camera urls."""
+        camera_data = self.get_camera(camera_id)
+        home_id = camera_data["home_id"]
+
+        if not camera_data or camera_data.get("status") == "disconnected":
+            self.cameras[home_id][camera_id]["local_url"] = None
+            self.cameras[home_id][camera_id]["vpn_url"] = None
+            return
+
+        vpn_url = camera_data.get("vpn_url")
+        if vpn_url and camera_data.get("is_local"):
+
+            async def async_check_url(url: str) -> Optional[str]:
+                try:
+                    resp = await self.auth.async_post_request(url=f"{url}/command/ping")
+                except ReadTimeout:
+                    LOG.debug("Timeout validation of camera url %s", url)
+                    return None
+                except ApiError:
+                    LOG.debug("Api error for camera url %s", url)
+                    return None
+
+                return resp.get("local_url") if resp else None
+
+            temp_local_url = await async_check_url(vpn_url)
+            if temp_local_url:
+                self.cameras[home_id][camera_id]["local_url"] = await async_check_url(
+                    temp_local_url,
+                )
+
+    async def async_set_persons_home(self, person_ids: List[str], home_id: str):
+        """Mark persons as home."""
+        post_params = {"home_id": home_id, "person_ids[]": person_ids}
+        return await self.auth.async_post_request(
+            url=_SETPERSONSHOME_REQ,
+            params=post_params,
+        )
+
+    async def async_set_persons_away(self, person_id: str, home_id: str):
+        """Mark a person as away or set the whole home to being empty."""
+        post_params = {"home_id": home_id, "person_id": person_id}
+        return await self.auth.async_post_request(
+            url=_SETPERSONSAWAY_REQ,
+            params=post_params,
+        )
+
+    async def async_get_live_snapshot(self, camera_id: str):
+        """Retrieve live snapshot from camera."""
+        local, vpn = self.camera_urls(camera_id)
+        if not local and not vpn:
+            return None
+        return await self.auth.async_post_request(
+            url=f"{(local or vpn)}/live/snapshot_720.jpg",
+            timeout=10,
+        )
+
+    async def async_get_camera_picture(
+        self,
+        image_id: str,
+        key: str,
+    ) -> Tuple[bytes, Optional[str]]:
+        """Download a specific image (of an event or user face) from the camera."""
+        post_params = {"image_id": image_id, "key": key}
+        resp = await self.auth.async_post_request(
+            url=_GETCAMERAPICTURE_REQ,
+            params=post_params,
+        )
+        image_type = imghdr.what("NONE.FILE", resp)
+        return resp, image_type
+
+    async def async_get_profile_image(
+        self,
+        name: str,
+    ) -> Tuple[Optional[bytes], Optional[str]]:
+        """Retrieve the face of a given person."""
+        for person in self.persons:
+            if name == self.persons[person].get("pseudo"):
+                image_id = self.persons[person]["face"]["id"]
+                key = self.persons[person]["face"]["key"]
+                return await self.async_get_camera_picture(image_id, key)
+
+        return None, None
